/*!
 * \file    utils.cpp
 * \brief   Utilities to handle matrix files, chrono, etc...
 *
 * \author
 *    Christos Choutouridis AEM:8997
 *    <cchoutou@ece.auth.gr>
 */

#include <utils.h>
#include <algorithm>

/*!
 * Initialize the matrix as Erdős-Rényi graph
 * \param A    The matrix to initialize
 * \param p    The probability of each edge
 */
void init_ER_graph (matrix& A, double p) {
   std::random_device rd;
   std::mt19937 gen(rd());
   std::binomial_distribution<> d(1, p);

#if CODE_VERSION == V12
   std::transform (A.begin(), A.end(), A.begin(),
         [&] (int x) { std::ignore =x; return d(gen); }
#else
   A.for_each_in(0, A.size(), [&](auto i) {
      A.for_each_in(i+1, A.size(), [&](auto j){
         matrix::dataType edge = d(gen);
         if (edge) {
            A.set(edge, i, j);
            A.set(edge, j, i);
         }
      });
   });
#endif

}

/*!
 * Utility to print the graph to sdtout
 */
void print_ER_graph (matrix& A) {
   matrix::indexType N = (A.size() < (matrix::indexType)session.print_size) ? A.size() : session.print_size;

   A.for_each_in(0, N, [&](auto i){
      A.for_each_in(0, N, [&](auto j) {
         std::cout << A(i, j) << ' ';
      });
      std::cout << '\n';
   });
}

